/*********************************************************************
Features: RFID READER receiving program
Description: Receive data display lights flash once, and displays
the ID value through the serial port on
**********************************************************************/

#include <reg24le1.h>
#include <stdint.h>
#include "API.h"

// #define PIN24
#define PIN32
// #define PIN48

#ifdef PIN24
sbit          S3 = P0 ^ 6;	// 1/0 = not pressed / Press
sbit          S2 = P0 ^ 4;	// 1/0 = not pressed / Press
sbit          S1 = P0 ^ 3;	// 1/0 = not pressed / Press

sbit          LED3 = P0 ^ 2;	// 1/0 = off / on
sbit          LED2 = P0 ^ 1;	// 1/0 = off / on
sbit          LED1 = P0 ^ 0;	// 1/0 = off / on

#define p0dir 0xd8
#define p1dir 0xff
#endif

#ifdef PIN32
sbit          S3 = P1 ^ 4;	// 1/0 = not pressed / Press
sbit          S2 = P1 ^ 3;	// 1/0 = not pressed / Press
sbit          S1 = P1 ^ 2;	// 1/0 = not pressed / Press

sbit          LED3 = P0 ^ 1;	// 1/0 = off / on
sbit          LED2 = P0 ^ 2;	// 1/0 = off / on
sbit          LED1 = P0 ^ 0;	// 1/0 = off / on

#define p0dir 0xf0
#define p1dir 0xff
#endif

#ifdef PIN48
sbit          S3 = P0 ^ 7;	// 1/0 = not pressed / Press
sbit          S2 = P0 ^ 6;	// 1/0 = not pressed / Press
sbit          S1 = P0 ^ 5;	// 1/0 = not pressed / Press

sbit          LED3 = P0 ^ 2;	// 1/0 = off / on
sbit          LED2 = P0 ^ 1;	// 1/0 = off / on
sbit          LED1 = P0 ^ 0;	// 1/0 = off / on

#define p0dir 0xf0
#define p1dir 0x02
#endif

#define INTERRUPT_RFIRQ 9

#define TX_ADR_WIDTH 5		// RF transceiver 5 bytes of address
#define TX_PLOAD_WIDTH 20	// packet length is 20 bytes

uint8_t const TX_ADDRESS[TX_ADR_WIDTH] = { 0x34, 0x56, 0x78, 0x90, 0x12 };	// define RF transceiver address

uint8_t data  rx_buf[TX_PLOAD_WIDTH];

uint8_t bdata sta;
sbit          RX_DR = sta ^ 6;
sbit          TX_DS = sta ^ 5;
sbit          MAX_RT = sta ^ 4;
/**************************************************
Function: Delay
**************************************************/
void
delay( uint16_t x )
{
  uint16_t      i, j;

  for ( i = 0; i < x; i++ )
  {
    j = 108;
    while ( j-- ) ;
  }
}

/**************************************************
Function: Hardware SPI read and write
**************************************************/
uint8_t
SPI_RW( uint8_t value )
{
  SPIRDAT = value;

  while ( !( SPIRSTAT & 0x02 ) ) ;	// wait for SPI transfer complete

  return SPIRDAT;		// returns the value read
}

/**************************************************
Function: write RF register, read RF state value
**************************************************/
uint8_t
SPI_RW_Reg( uint8_t reg, uint8_t value )
{
  uint8_t       status;

  RFCSN = 0;
  status = SPI_RW( reg );	// Select the RF register
  SPI_RW( value );		// write data
  RFCSN = 1;

  return ( status );		// returns the RF state value
}

/**************************************************
Function: Reading RF register
**************************************************/
uint8_t
SPI_Read( uint8_t reg )
{
  uint8_t       reg_val;

  RFCSN = 0;
  SPI_RW( reg );		// Select the RF register
  reg_val = SPI_RW( 0 );	// read data
  RFCSN = 1;

  return ( reg_val );		// returns the RF state value
}

/**************************************************
Features: Multi-byte read RF data to the buffer register
**************************************************/
uint8_t
SPI_Read_Buf( uint8_t reg, uint8_t * pBuf, uint8_t bytes )
{
  uint8_t       status, byte_ctr;

  RFCSN = 0;
  status = SPI_RW( reg );	// Select the RF register

  for ( byte_ctr = 0; byte_ctr < bytes; byte_ctr++ )
    pBuf[byte_ctr] = SPI_RW( 0 );	// read data connection

  RFCSN = 1;

  return ( status );		// returns the RF state value
}

/**************************************************
Function: Multi-byte data buffer is written to RF register
**************************************************/
uint8_t
SPI_Write_Buf( uint8_t reg, uint8_t * pBuf, uint8_t bytes )
{
  uint8_t       status, byte_ctr;

  RFCSN = 0;
  status = SPI_RW( reg );	// Select the RF register
  for ( byte_ctr = 0; byte_ctr < bytes; byte_ctr++ )	// write data connection
    SPI_RW( *pBuf++ );
  RFCSN = 1;
  return ( status );		// returns the RF state value
}

/**************************************************
Function: set to receive mode
**************************************************/
void
RX_Mode( void )
{
  RFCE = 0;
  SPI_RW_Reg( WRITE_REG + CONFIG, 0x0f );	// on power, CRC is 2 bytes, the receive mode, an interrupt is generated to allow RX_DR
  RFCE = 1;			// start receiving mode
}

/**************************************************
Function: RF initialization
**************************************************/
void
rf_init( void )
{
  RFCE = 0;			// RF Close
  RFCKEN = 1;			// start the RF clock
  RF = 1;			// allows the RF interrupt
  delay( 1000 );
  SPI_Write_Buf( WRITE_REG + TX_ADDR, TX_ADDRESS, TX_ADR_WIDTH );	// Set transmitter address length
  SPI_Write_Buf( WRITE_REG + RX_ADDR_P0, TX_ADDRESS, TX_ADR_WIDTH );	// Set the receiver address length
  SPI_RW_Reg( WRITE_REG + EN_AA, 0x01 );	// start automatic answer function
  SPI_RW_Reg( WRITE_REG + EN_RXADDR, 0x01 );	// PIPE0 receive data
  SPI_RW_Reg( WRITE_REG + SETUP_RETR, 0x1a );	// automatic repeat 10 times
  SPI_RW_Reg( WRITE_REG + RF_CH, 40 );	// RF frequency 2440MHz
  SPI_RW_Reg( WRITE_REG + RF_SETUP, 0x0f );	// transmit power 0dBm, the transmission rate 2Mbps,
  SPI_RW_Reg( WRITE_REG + RX_PW_P0, TX_PLOAD_WIDTH );	// PIPE0 receive packet length
}

/**************************************************
Function: RF interrupt service routine
**************************************************/
void
RF_IRQ( void )
interrupt     INTERRUPT_RFIRQ
{
  sta = SPI_Read( STATUS );	// read status values

  if ( RX_DR )
  {
    SPI_Read_Buf( RD_RX_PLOAD, rx_buf, TX_PLOAD_WIDTH );	// read data FIFO
    SPI_RW_Reg( FLUSH_RX, 0 );	// Clear the RX FIFO
  }
  SPI_RW_Reg( WRITE_REG + STATUS, 0x70 );	// clear all interrupt flags
}

/**************************************************
Function: serial port initialization subroutine
Description: baud 19.2K, using the internal baud rate generator
**************************************************/
void
uart_init( void )
{
  ES0 = 0;			// Off UART0 interrupt
  REN0 = 1;			// allowed to receive
  SM0 = 0;			// serial mode 1,8 bit variable baud rate
  SM1 = 1;
  PCON | = 0x80;		// SMOD = 1
  ADCON | = 0x80;		// Select the internal baud rate generator

  S0RELL = 0xe6;		// Baud Rate 19.2K (decimal 998 = hex 0x03e6)
  S0RELH = 0x03;
  TI0 = 0;			// clear to send complete flag
  S0BUF = 0x99;			// send initial
}

/**************************************************
Function: send 1 byte of data to the serial port
**************************************************/
void
uart_putchar( uint8_t x )
{
  while ( !TI0 ) ;		// wait for the transmission completion
  TI0 = 0;			// clear to send complete flag
  S0BUF = x;			// send data
}

/**************************************************
Function: I / O port initialization
**************************************************/
void
io_init( void )
{
  P0DIR = p0dir;
  // set the I / O port input and output P1DIR = p1dir;

  LED1 = LED2 = LED3 = 0;	// full bright light
  delay( 1000 );
  LED1 = LED2 = LED3 = 1;	// Light Quanmie
}

/**************************************************
Function: main
**************************************************/
void
main( void )
{
  io_init(  );			// I / O port initialization
  uart_init(  );		// serial port initialization
  rf_init(  );			// RF initialization
  EA = 1;			// enable interrupt

  RX_Mode(  );			// receive mode

  while ( 1 )
  {
    if ( RX_DR )		// data has been received
    {
      sta = 0;

      LED2 = 0;

      uart_putchar( rx_buf[0] );	// show all ID values
      uart_putchar( rx_buf[1] );
      uart_putchar( rx_buf[2] );
      uart_putchar( rx_buf[3] );
      uart_putchar( rx_buf[4] );
      uart_putchar( rx_buf[5] );

      delay( 1000 );

      LED1 = LED2 = LED3 = 1;	// Light Quanmie
    }
  }
}
